/***************************************************************************
                          cclient.h  -  description
                             -------------------
    begin                : Sun Sep 30 2001
    copyright            : (C) 2001-2003 by Mathias Kster
    email                : mathen@ketelhot.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CCLIENT_H
#define CCLIENT_H

/**
  *@author Mathias Kster
  */

#include <dclib/dcos.h>
#include <dclib/core/cstring.h>
#include <dclib/core/ccallback.h>
#include <dclib/dcobject.h>
#include <dclib/cdcproto.h>
#include <dclib/cclientssl.h>

/** */
typedef enum eReconnectState {
	ersNONE,
	ersFORCEMOVE,
	ersDISABLED,
	ersENABLED
} eReconnectState;

class CClient;
class CObject;
class CStringList;
class CThreadStringList;
class CMessageNickList;
class CMessageMyInfo;

class DLL_EXPORT CClient : public CDCProto {

public:
	/** */
	CClient();
	/** */
	virtual ~CClient();

	/** */
	void SetNick( CString nick );
	/** */
	CString GetNick() const;
	/** */
	void SetEMail( CString email );
	/** */
	CString GetEMail() const;
	/** */
	void SetConnectionType( CString ctype );
	/** */
	CString GetConnectionType() const;
	/** */
	void SetComment( CString comment );
	/** */
	CString GetComment() const;
	/** */
	void SetHubName( CString hubname );
	/** */
	CString GetHubName() const;
	/** */
	void SetShareSize( CString sharesize );
	/** */
	CString GetShareSize() const;
	/** */
	void SetVersion( CString version );
	/** */
	CString GetVersion() const;
	/** */
	void SetMode( eClientMode mode );
	/** */
	eClientMode GetMode() const;
	/** */
	void SetAwayMode( eUserAwayMode mode );
	/** */
	eUserAwayMode GetAwayMode() const;
	/** */
	bool IsHandshake() const;
	/** */
	bool IsAdmin() const;
	/** */
	bool IsAdmin( const CString & nick ) const;
	/** */
	eUserClientVersion GetUserClientVersion( const CString & nick ) const;

	/** */
	void SetHandleUserList( bool b );
	/** */
	void SetHandleSearch( bool b );
	/** */
	void SetHandleMyinfo( bool b );
	/** */
	void SetHandleForceMove( bool b );
	/** */
	void SetHandleTransfer( bool b );

	/** */
	ulonglong GetHubShareSize() const;

	/** user list functions */
	void InitUserList( CMessageNickList * NickList );
	/** */
	void InitOperatorList( CMessageNickList * NickList );
	/** */
	void AppendUser( CString & nick, bool getinfo = TRUE );
	/** */
	void RemoveUser( CString & nick );
	/** */
	bool UpdateUser( CMessageMyInfo * pMyInfo );
	/** */
	void UpdateUserSlots( CMessageSearchResult * pSearchResult );

	/** */
	bool IsUserOnline( CString & nick );

	/** */
	void SetCallBackFunction( _CCallback * callback );

	/** */
	int HandleMessage( const CString & message );

	/** send private message */
	virtual int SendPrivateMessage( CString sNick, CString sTo, CString sMsg, CString sFromNick = "" );
	/** */
	virtual int RequestNickList();

	/** */
	virtual void DataAvailable( const char * buffer, int len );
	/** */
	virtual void DataTimeout();
	/** */
	virtual void Notify();
	/** */
	virtual void ConnectionState( eConnectionState state );
	/** callback function */
	virtual int DC_CallBack( CObject * ) { return -1; };

	/** */
	int Disconnect( bool force = FALSE );

protected:
	/** */
	void UpdateReconnect( eReconnectState state, int count = -1 );

private:
	/** */
	CClientSSL m_pMessageSSL;
	/** */
	CString m_sNick;
	/** */
	CString m_sEMail;
	/** */
	CString m_sConnectionType;
	/** */
	CString m_sComment;
	/** */
	CString m_sHubName;
	/** */
	CString m_sShareSize;
	/** */
	CString m_sVersion;
	/** */
	eClientMode m_ecMode;
	/** */
	bool m_bHandshake;
	/** */
	bool m_bHandleUserList;
	/** */
	bool m_bHandleSearch;
	/** */
	bool m_bHandleMyinfo;
	/** */
	bool m_bHandleForceMove;
	/** */
	bool m_bHandleTransfer;
	/** */
	bool m_bUpdateMyinfo;
	/** */
	time_t m_timeMyinfoTimeout;
	/** */
	eUserAwayMode m_eAwayMode;
	/** */
	ulonglong m_nHubShareSize;
	/** */
	CThreadStringList * m_pUserList;
	/** */
	CString m_sBuffer;
	/** callback function */
	_CCallback * m_pCallback;
	/** */
	CThread m_ClientThread;
	/** */
	time_t m_timeReconnectTimeout;
	/** */
	eReconnectState m_eReconnectState;
	/** */
	int m_nReconnectCount;
	/** */
	int m_nNickListHandler;
	/** */
	int m_nOpListHandler;
	/** */
	CMessageCapabilities m_MessageCapabilities;
};

/** inline functions */

/** */
inline void CClient::SetNick( CString nick )
{ m_ClientThread.Lock(); if ( m_sNick != nick ) { m_bUpdateMyinfo = TRUE; m_sNick=nick; } m_ClientThread.UnLock(); }
/** */
inline CString CClient::GetNick() const
{ return m_sNick; }
/** */
inline void CClient::SetEMail( CString email )
{ m_ClientThread.Lock(); if ( m_sEMail != email ) { m_bUpdateMyinfo = TRUE; m_sEMail=email; } m_ClientThread.UnLock(); }
/** */
inline CString CClient::GetEMail() const
{ return m_sEMail; }
/** */
inline void CClient::SetConnectionType( CString ctype )
{ m_ClientThread.Lock(); if ( m_sConnectionType != ctype ) { m_bUpdateMyinfo = TRUE; m_sConnectionType=ctype; } m_ClientThread.UnLock(); }
/** */
inline CString CClient::GetConnectionType() const
{ return m_sConnectionType; }
/** */
inline void CClient::SetComment( CString comment )
{ m_ClientThread.Lock(); if ( m_sComment != comment ) { m_bUpdateMyinfo = TRUE; m_sComment=comment; } m_ClientThread.UnLock(); }
/** */
inline CString CClient::GetComment() const
{ return m_sComment; }
/** */
inline void CClient::SetHubName( CString hubname )
{ m_ClientThread.Lock(); m_sHubName = hubname; m_ClientThread.UnLock(); }
/** */
inline CString CClient::GetHubName() const
{ return m_sHubName; }
/** */
inline void CClient::SetShareSize( CString sharesize )
{ m_ClientThread.Lock(); if ( m_sShareSize != sharesize ) { m_bUpdateMyinfo = TRUE; m_sShareSize = sharesize; } m_ClientThread.UnLock(); }
/** */
inline CString CClient::GetShareSize() const
{ return m_sShareSize; }
/** */
inline void CClient::SetVersion( CString version )
{ m_ClientThread.Lock(); m_sVersion = version; m_ClientThread.UnLock(); }
/** */
inline CString CClient::GetVersion() const
{ return m_sVersion; }
/** */
inline void CClient::SetMode( eClientMode mode )
{ m_ClientThread.Lock(); m_ecMode=mode; m_ClientThread.UnLock(); }
/** */
inline eClientMode CClient::GetMode() const
{ return m_ecMode; }
/** */
inline bool CClient::IsHandshake() const
{ return m_bHandshake; }
/** */
inline void CClient::SetCallBackFunction( _CCallback * callback )
{ m_ClientThread.Lock(); if ( m_pCallback ) delete m_pCallback; m_pCallback = callback; m_ClientThread.UnLock(); }
/** */
inline void CClient::SetHandleUserList( bool b )
{ m_ClientThread.Lock(); m_bHandleUserList = b; m_ClientThread.UnLock(); }
/** */
inline void CClient::SetHandleSearch( bool b )
{ m_ClientThread.Lock(); m_bHandleSearch = b; m_ClientThread.UnLock(); }
/** */
inline void CClient::SetHandleMyinfo( bool b )
{ m_ClientThread.Lock(); m_bHandleMyinfo = b; m_ClientThread.UnLock(); }
/** */
inline void CClient::SetHandleForceMove( bool b )
{ m_ClientThread.Lock(); m_bHandleForceMove = b; m_ClientThread.UnLock(); }
/** */
inline void CClient::SetHandleTransfer( bool b )
{ m_ClientThread.Lock(); m_bHandleTransfer = b; m_ClientThread.UnLock(); }
/** */
inline ulonglong CClient::GetHubShareSize() const
{ return m_nHubShareSize; }
/** */
inline void CClient::SetAwayMode( eUserAwayMode mode )
{ m_ClientThread.Lock(); m_eAwayMode = mode; m_ClientThread.UnLock(); }
/** */
inline eUserAwayMode CClient::GetAwayMode() const
{ return m_eAwayMode; }

#endif
