/***************************************************************************
                          chubsearch.h  -  description
                             -------------------
    begin                : Fri Mar 15 2002
    copyright            : (C) 2002-2003 by Mathias Kster
    email                : mathen@ketelhot.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CHUBSEARCH_H
#define CHUBSEARCH_H

/**
  *@author Mathias Kster
  */

#include <time.h>

#include <dclib/dcos.h>
#include <dclib/core/cstring.h>
#include <dclib/core/cconnection.h>
#include <dclib/core/csingleton.h>

typedef enum eHubSearchType {
	ehstNONE=0,
	ehstLOCAL,
	ehstGLOBAL,
	ehstWAITTIMEOUT,
	ehstSTOP,
	ehstEXTERNAL
} eHubSearchType;

typedef enum eHubSearchHubs {
	ehshCONNECTEDSINGLE=0,
	ehshCONNECTEDALL,
	ehshPUBLIC,
	ehshBOOKMARK
} eHubSearchHubs;

class CClient;
class CHubSearch;
class CStringList;

class DLL_EXPORT sHubSearchClient : public CObject {
public:
	/** */
	sHubSearchClient() {
		m_pClient = 0;
		m_bRemove = FALSE;
		m_bEnableSearch = FALSE;
		m_tHubTimeout = 0;
		m_pCurrentSearch = 0;
	};
	/** */
	virtual ~sHubSearchClient() {};

	/** */
	CClient * m_pClient;
	/** */
	bool m_bRemove;
	/** */
	time_t m_tHubTimeout;
	/** */
	bool m_bEnableSearch;
	/** */
	CObject * m_pCurrentSearch;
	/** */
	time_t m_tSearchTimeout;
};

#ifdef WIN32
template class DLL_EXPORT CSingleton<CHubSearch>;
#endif

class CMessageSearchResult;
class CMessageSearchFile;

class DLL_EXPORT CHubSearch : public CSingleton<CHubSearch>, public CConnection {
public:
	/** */
	CHubSearch();
	/** */
	virtual ~CHubSearch();

	/** callback function */
	virtual int DC_CallBack( CObject * ) { return -1; };
	/** */
	virtual void DataAvailable( const char * buffer, int len );
	/** */
	virtual void ConnectionState( eConnectionState state );

	/** */
	bool ExternalSearch() { return (m_ehSearchType == ehstEXTERNAL); }
	/** */
	bool ExternalSearch( bool enabled );

	/** */
	int StartSearch( eHubSearchHubs hubs, bool multi, CList<CObject> * searchlist, CStringList * hublist, CString s = "" );
	/** */
	void StopSearch();

	/** */
	int HandleMessage( char * c, int len );
	/** passive search results */
	bool HandleMessage( CMessageSearchResult * MessageSearchResult );
	/** */
	int DC_ClientCallBack( CObject * Client, CObject * Object );
	/** */
	int DC_ManagerCallBack( CObject *, CObject * );

	/** */
	CString GetSearchString( CMessageSearchFile * msg, bool multi = FALSE );

protected:
	/** */
	eHubSearchType m_ehSearchType;
	/** store clients */
	CList<CObject> * m_pHubSearchClientList;
	/** */
	CStringList * m_pHubServerList;
	/** */
	int m_nMaxThreads;
	/** */
	time_t m_tStartTime;
	/** */
	long m_nError;
	/** */
	long m_nCurrentHub;
	/** */
	bool m_bEnableTag;

private:
	/** */
	friend class CSingleton<CHubSearch>;
	/** */
	void SendDebug( CString s );
	/** */
	void SendSearchResult( CObject * object );
	/** */
	void CheckClient();
	/** mark client as removeable */
	void RemoveClient( CClient * Client );
	/** enable search for this client */
	void EnableSearch( CClient * Client );
	/** create new client connection */
	void NewClient();
	/** send the search to this client */
	bool SendSearch( sHubSearchClient * HubSearchClient );
	/** send the search to this client */
	int SendSearch( CString s );
	/** */
	void UpdateClients();

	/** */
	_CCallback * m_pCallback;

	/** */
	CString m_sBuffer;
	/** */
	CThread ClientThread;
	/** */
	CThread HubSearchClientListThread;
	/** */
	CString * m_sCurrentServer;
	/** */
	CList<CObject> * m_pSearchList;
	/** */
	bool m_bHandleUserList;
	/** */
	time_t m_tHubSearchTimeout;
	/** */
	CString m_sHubName;
	/** */
	CObject * m_pCurrentSearchObject;
	/** */
	bool m_bMultiSearch;
};

#endif
