/***************************************************************************
                          cservermanager.h  -  description
                             -------------------
    begin                : Don Mai 16 2002
    copyright            : (C) 2002-2003 by Mathias Kster
    email                : mathen@ketelhot.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CSERVERMANAGER_H
#define CSERVERMANAGER_H

/**
  *@author Mathias Kster
  */

#include <dclib/dcos.h>
#include <dclib/core/cobject.h>
#include <dclib/core/cstring.h>
#include <dclib/core/cthread.h>
#include <dclib/core/csingleton.h>

typedef enum eHubState {
	ehsNONE,
	ehsONLINE,
	ehsOFFLINE
};

class CClient;
class DCHubObject;
class CServerManager;
class CMessageSearchFile;
class CStringList;
class CHttp;
class DCConfigHubListUrl;
class CByteArray;

#ifdef WIN32
template class DLL_EXPORT CSingleton<CServerManager>;
#endif

class DLL_EXPORT CServerManager : public CSingleton<CServerManager> {
public:
	/** */
	CServerManager();
	/** */
	virtual ~CServerManager();

	/** */
	void ConnectClient( CString hubname, CString server );

	/** send message to all connected servers */
	int SendStringToConnectedServers( CString s, CString hubname = "" );
	/** send a search string (extracted from message) to all connected servers */
	int SendStringToConnectedServers( CMessageSearchFile *sf );
	/** send myinfo to all connected servers */
	int SendMyInfoToConnectedServers();
	/** 0: send -1: user offline -2: hub not ready (connection state/handshake) -3: hub offline -4: send error */
	int SendConnectionRequest( CString nick, CString hubname );
	/** */
	bool IsUserOnline( CString nick, CString hubname, CList<DCHubObject> * list = 0 );
	/** */
	eHubState IsHubOnline( CString hubname );
	/** */
	bool IsAdmin( CString hubname, CString nick = "" );
	/** */
	CString GetHubHost( CString hubname );

	/** */
	long GetConnectedHubCount( bool admin = FALSE );

	/** */
	CStringList * GetPublicHubServerList();
	/** */
	CStringList * GetBookmarkHubServerList();
	/** */
	CStringList * GetConnectedHubServerList();

	/** callback function */
	virtual int DC_CallBack( CObject * ) { return -1; };

protected:
	/** */
	void AddHub( CClient * client );
	/** */
	void RemoveHub( CClient * client );

	/** */
	bool GetPublicHubList();
	/** */
	bool NextHubListUrl();

	/** */
	void Connect( CString hubname, CString server, CClient * client );

	/** */
	CClient * GetHub( CString hubname );

	/** */
	CThread m_Thread;
	/** client list */
	CThreadList<CClient> * m_pClientList;

	/** */
	CHttp * m_pHttp;
	/** */
	CList<DCConfigHubListUrl> * m_pHubListUrlList;
	/** */
	DCConfigHubListUrl * m_pHubListUrl;
	/** */
	CByteArray * m_pHubListData;

private:
	/** */
	friend class CSingleton<CServerManager>;

	/** http callback function */
	int HttpCallBack( CObject *, CObject * );
	/** thread callbackfunction */
	int Callback( CObject * , CObject * );
	/** */
	int ParsePublicHubList( CObject * , CObject * );

	/** */
	int SM_ClientCallBack( CObject * Sender, CObject * Object );

	/** */
	time_t m_nReloadHubListTimeout;
	/** */
	bool m_bGetHubListDone;

	/** */
	_CCallback * m_pCallback;
};

#endif
